'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _deleteAtRange = require('./delete-at-range');

var _deleteAtRange2 = _interopRequireDefault(_deleteAtRange);

var _uid = require('../../utils/uid');

var _uid2 = _interopRequireDefault(_uid);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Insert a `fragment` at a `range`.
 *
 * @param {State} state
 * @param {Selection} range
 * @param {Document} fragment
 * @return {State} state
 */

function insertFragmentAtRange(state, range, fragment) {
  var _state = state;
  var document = _state.document;

  // Ensure that the selection is normalized.

  range = range.normalize(document);

  // If the range is expanded, delete first.
  if (range.isExpanded) {
    state = (0, _deleteAtRange2.default)(state, range);
    document = state.document;
    range = range.collapseToStart();
  }

  // If the fragment is empty, do nothing.
  if (!fragment.length) return state;

  // Make sure each node in the fragment has a unique key.
  fragment = fragment.mapDescendants(function (child) {
    return child.set('key', (0, _uid2.default)());
  });

  // Split the inlines if need be.
  if (!document.isInlineSplitAtRange(range)) {
    document = document.splitInlineAtRange(range);
  }

  // Determine the start and next children to insert into.
  var _range = range;
  var startKey = _range.startKey;
  var endKey = _range.endKey;

  var block = document.getClosestBlock(startKey);
  var start = document.getDescendant(startKey);
  var startChild = void 0;
  var nextChild = void 0;

  if (range.isAtStartOf(document)) {
    nextChild = document.getClosestBlock(document.getTexts().first());
  }

  if (range.isAtStartOf(block)) {
    nextChild = block.getHighestChild(block.getTexts().first());
  } else if (range.isAtStartOf(start)) {
    startChild = block.getHighestChild(block.getPreviousText(start));
    nextChild = block.getNextSibling(startChild);
  } else {
    startChild = block.getHighestChild(start);
    nextChild = block.getNextSibling(startChild);
  }

  // Get the first and last block of the fragment.
  var blocks = fragment.getBlocks();
  var firstBlock = blocks.first();
  var lastBlock = blocks.last();

  // If the block is empty, merge in the first block's type and data.
  if (block.length == 0) {
    block = block.merge({
      type: firstBlock.type,
      data: firstBlock.data
    });
  }

  // Insert the first blocks nodes into the starting block.
  if (startChild) {
    block = block.insertChildrenAfter(startChild, firstBlock.nodes);
  } else {
    block = block.insertChildrenBefore(nextChild, firstBlock.nodes);
  }

  document = document.updateDescendant(block);

  // If there are no other siblings, that's it.
  if (firstBlock == lastBlock) {
    document = document.normalize();
    state = state.merge({ document: document });
    return state;
  }

  // Otherwise, remove the fragment's first block's highest solo parent...
  var highestParent = fragment.getHighestOnlyChildParent(firstBlock);
  fragment = fragment.removeDescendant(highestParent || firstBlock);

  // Then, add the inlines after the cursor from the current block to the
  // start of the last block in the fragment.
  if (nextChild) {
    lastBlock = lastBlock.concatChildren(block.getChildrenAfterIncluding(nextChild));
    fragment = fragment.updateDescendant(lastBlock);

    block = block.removeChildrenAfterIncluding(nextChild);
    document = document.updateDescendant(block);
  }

  // Finally, add the fragment's children after the block.
  document = document.insertChildrenAfter(block, fragment.nodes);
  document = document.normalize();
  state = state.merge({ document: document });
  return state;
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = insertFragmentAtRange;