'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _immutable = require('immutable');

var _normalizeNodeOrMarkProperties = require('../../utils/normalize-node-or-mark-properties');

var _normalizeNodeOrMarkProperties2 = _interopRequireDefault(_normalizeNodeOrMarkProperties);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Unwrap all of the block nodes in a `range` from a block with `properties`.
 *
 * @param {State} state
 * @param {Selection} range
 * @param {String or Object} properties
 * @return {State} state
 */

function unwrapBlockAtRange(state, range, properties) {
  properties = (0, _normalizeNodeOrMarkProperties2.default)(properties);
  var _state = state;
  var document = _state.document;

  // Get the deepest blocks in the range.

  var blocks = document.getBlocksAtRange(range);

  // Get the matching wrapper blocks.
  var wrappers = blocks.reduce(function (memo, text) {
    var match = document.getClosest(text, function (parent) {
      if (parent.kind != 'block') return false;
      if (properties.type && parent.type != properties.type) return false;
      if (properties.data && !parent.data.isSuperset(properties.data)) return false;
      return true;
    });

    if (match) memo = memo.add(match);
    return memo;
  }, new _immutable.Set());

  // For each of the wrapper blocks...
  wrappers.forEach(function (wrapper) {
    var first = wrapper.nodes.first();
    var last = wrapper.nodes.last();
    var parent = document.getParent(wrapper);

    // Get the wrapped direct children.
    var children = wrapper.nodes.filter(function (child) {
      return blocks.some(function (block) {
        return child == block || child.hasDescendant(block);
      });
    });

    // Determine what the new nodes should be...
    var firstMatch = children.first();
    var lastMatch = children.last();
    var nodes = void 0;

    // If the first and last both match, remove the wrapper completely.
    if (first == firstMatch && last == lastMatch) {
      nodes = parent.nodes.takeUntil(function (n) {
        return n == wrapper;
      }).concat(wrapper.nodes).concat(parent.nodes.skipUntil(function (n) {
        return n == wrapper;
      }).rest());
    }

    // If only the last child matches, move the last nodes.
    else if (last == lastMatch) {
        var remain = wrapper.nodes.takeUntil(function (n) {
          return n == firstMatch;
        });
        var updated = wrapper.merge({ nodes: remain });
        nodes = parent.nodes.takeUntil(function (n) {
          return n == wrapper;
        }).push(updated).concat(children).concat(parent.nodes.skipUntil(function (n) {
          return n == wrapper;
        }).rest());
      }

      // If only the first child matches, move the first ones.
      else if (first == firstMatch) {
          var _remain = wrapper.nodes.skipUntil(function (n) {
            return n == lastMatch;
          }).rest();
          var _updated = wrapper.merge({ nodes: _remain });
          nodes = parent.nodes.takeUntil(function (n) {
            return n == wrapper;
          }).concat(children).push(_updated).concat(parent.nodes.skipUntil(function (n) {
            return n == wrapper;
          }).rest());
        }

        // Otherwise, move the middle ones.
        else {
            var firsts = wrapper.nodes.takeUntil(function (n) {
              return n == firstMatch;
            });
            var lasts = wrapper.nodes.skipUntil(function (n) {
              return n == lastMatch;
            }).rest();
            var updatedFirst = wrapper.merge({ nodes: firsts });
            var updatedLast = wrapper.merge({ nodes: lasts });
            nodes = parent.nodes.takeUntil(function (n) {
              return n == wrapper;
            }).push(updatedFirst).concat(children).push(updatedLast).concat(parent.nodes.skipUntil(function (n) {
              return n == wrapper;
            }).rest());
          }

    document = parent == document ? document.merge({ nodes: nodes }) : document.updateDescendant(parent.merge({ nodes: nodes }));
  });

  document = document.normalize();
  state = state.merge({ document: document });
  return state;
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = unwrapBlockAtRange;