'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _inline = require('../../models/inline');

var _inline2 = _interopRequireDefault(_inline);

var _normalizeNodeOrMarkProperties = require('../../utils/normalize-node-or-mark-properties');

var _normalizeNodeOrMarkProperties2 = _interopRequireDefault(_normalizeNodeOrMarkProperties);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Wrap the text and inlines in a `range` in a new inline with `properties`.
 *
 * @param {State} state
 * @param {Selection} range
 * @param {String or Object} properties
 * @return {State} state
 */

function wrapInlineAtRange(state, range, properties) {
  properties = (0, _normalizeNodeOrMarkProperties2.default)(properties);
  var _state = state;
  var document = _state.document;

  // If collapsed, there's nothing to wrap.

  if (range.isCollapsed) return state;

  // Split at the start of the range.
  var start = range.collapseToStart();
  document = document.splitInlineAtRange(start);

  // Determine the new end of the range, and split there.
  var _range = range;
  var startKey = _range.startKey;
  var startOffset = _range.startOffset;
  var endKey = _range.endKey;
  var endOffset = _range.endOffset;

  var firstNode = document.getDescendant(startKey);
  var nextNode = document.getNextText(startKey);
  var end = startKey != endKey ? range.collapseToEnd() : Selection.create({
    anchorKey: nextNode.key,
    anchorOffset: endOffset - startOffset,
    focusKey: nextNode.key,
    focusOffset: endOffset - startOffset
  });

  document = document.splitInlineAtRange(end);

  // Calculate the new range to wrap around.
  var endNode = document.getDescendant(end.anchorKey);
  range = Selection.create({
    anchorKey: nextNode.key,
    anchorOffset: 0,
    focusKey: endNode.key,
    focusOffset: endNode.length
  });

  // Get the furthest inline nodes in the range.
  var texts = document.getTextsAtRange(range);
  var children = texts.map(function (text) {
    return document.getFurthestInline(text) || text;
  });

  // Iterate each of the child nodes, wrapping them.
  children.forEach(function (child) {
    var wrapper = _inline2.default.create({
      nodes: [child],
      type: properties.type,
      data: properties.data
    });

    // Replace the child in it's parent with the wrapper.
    var parent = document.getParent(child);
    var nodes = parent.nodes.takeUntil(function (n) {
      return n == child;
    }).push(wrapper).concat(parent.nodes.skipUntil(function (n) {
      return n == child;
    }).rest());

    // Update the parent.
    document = parent == document ? document.merge({ nodes: nodes }) : document.updateDescendant(parent.merge({ nodes: nodes }));
  });

  document = document.normalize();
  state = state.merge({ document: document });
  return state;
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = wrapInlineAtRange;