'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.applyOperation = applyOperation;

var _uid = require('../utils/uid');

var _uid2 = _interopRequireDefault(_uid);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Operations.
 *
 * @type {Object}
 */

var OPERATIONS = {
  // Text operations.
  insert_text: insertText,
  remove_text: removeText,
  // Mark operations.
  add_mark: addMark,
  remove_mark: removeMark,
  set_mark: setMark,
  // Node operations.
  insert_node: insertNode,
  move_node: moveNode,
  remove_node: removeNode,
  set_node: setNode,
  split_node: splitNode,
  // Selection operations.
  set_selection: setSelection
};

/**
 * Apply an `operation` to the current state.
 *
 * @param {Transform} transform
 * @param {Object} operation
 * @return {Transform}
 */

function applyOperation(transform, operation) {
  var state = transform.state;
  var operations = transform.operations;
  var type = operation.type;

  var fn = OPERATIONS[type];

  if (!fn) {
    throw new Error('Unknown operation type: "' + type + '".');
  }

  transform.state = fn(state, operation);
  transform.operations = operations.concat([operation]);
  return transform;
}

/**
 * Add mark to text at `offset` and `length` in node by `path`.
 *
 * @param {State} state
 * @param {Object} operation
 * @return {State}
 */

function addMark(state, operation) {
  var path = operation.path;
  var offset = operation.offset;
  var length = operation.length;
  var mark = operation.mark;
  var _state = state;
  var document = _state.document;

  var node = document.assertPath(path);
  node = node.addMark(offset, length, mark);
  document = document.updateDescendant(node);
  state = state.merge({ document: document });
  return state;
}

/**
 * Insert a `node` at `index` in a node by `path`.
 *
 * @param {State} state
 * @param {Object} operation
 * @return {State}
 */

function insertNode(state, operation) {
  var path = operation.path;
  var index = operation.index;
  var node = operation.node;
  var _state2 = state;
  var document = _state2.document;

  var parent = document.assertPath(path);
  var isParent = document == parent;
  var nodes = parent.nodes.splice(index, 0, node);
  parent = parent.merge({ nodes: nodes });
  document = isParent ? parent : document.updateDescendant(parent);
  state = state.merge({ document: document });
  return state;
}

/**
 * Insert `text` at `offset` in node by `path`.
 *
 * @param {State} state
 * @param {Object} operation
 * @return {State}
 */

function insertText(state, operation) {
  var path = operation.path;
  var offset = operation.offset;
  var text = operation.text;
  var marks = operation.marks;
  var _state3 = state;
  var document = _state3.document;

  var node = document.assertPath(path);
  node = node.insertText(offset, text, marks);
  document = document.updateDescendant(node);
  state = state.merge({ document: document });
  return state;
}

/**
 * Move a node by `path` to a new parent by `path` and `index`.
 *
 * @param {State} state
 * @param {Object} operation
 * @return {State}
 */

function moveNode(state, operation) {
  var path = operation.path;
  var newPath = operation.newPath;
  var newIndex = operation.newIndex;
  var _state4 = state;
  var document = _state4.document;

  var node = document.assertPath(path);

  var parent = document.getParent(node);
  var isParent = document == parent;
  var index = parent.nodes.indexOf(node);
  parent = parent.removeNode(index);
  document = isParent ? parent : document.updateDescendant(parent);

  var target = document.assertPath(newPath);
  var isTarget = document == target;
  target = target.insertNode(newIndex, node);
  document = isTarget ? target : document.updateDescendant(target);

  state = state.merge({ document: document });
  return state;
}

/**
 * Remove mark from text at `offset` and `length` in node by `path`.
 *
 * @param {State} state
 * @param {Object} operation
 * @return {State}
 */

function removeMark(state, operation) {
  var path = operation.path;
  var offset = operation.offset;
  var length = operation.length;
  var mark = operation.mark;
  var _state5 = state;
  var document = _state5.document;

  var node = document.assertPath(path);
  node = node.removeMark(offset, length, mark);
  document = document.updateDescendant(node);
  state = state.merge({ document: document });
  return state;
}

/**
 * Remove a node by `path`.
 *
 * @param {State} state
 * @param {Object} operation
 * @return {State}
 */

function removeNode(state, operation) {
  var path = operation.path;
  var _state6 = state;
  var document = _state6.document;

  var node = document.assertPath(path);
  var parent = document.getParent(node);
  var index = parent.nodes.indexOf(node);
  var isParent = document == parent;
  parent = parent.removeNode(index);
  document = isParent ? parent : document.updateDescendant(parent);
  document = document.normalize();
  state = state.merge({ document: document });
  return state;
}

/**
 * Remove text at `offset` and `length` in node by `path`.
 *
 * @param {State} state
 * @param {Object} operation
 * @return {State}
 */

function removeText(state, operation) {
  var path = operation.path;
  var offset = operation.offset;
  var length = operation.length;
  var _state7 = state;
  var document = _state7.document;

  var node = document.assertPath(path);
  node = node.removeText(offset, length);
  document = document.updateDescendant(node);
  document = document.normalize();
  state = state.merge({ document: document });
  return state;
}

/**
 * Set `properties` on mark on text at `offset` and `length` in node by `path`.
 *
 * @param {State} state
 * @param {Object} operation
 * @return {State}
 */

function setMark(state, operation) {
  var path = operation.path;
  var offset = operation.offset;
  var length = operation.length;
  var mark = operation.mark;
  var properties = operation.properties;
  var _state8 = state;
  var document = _state8.document;

  var node = document.assertPath(path);
  node = node.updateMark(offset, length, mark, properties);
  document = document.updateDescendant(node);
  state = state.merge({ document: document });
  return state;
}

/**
 * Set `properties` on a node by `path`.
 *
 * @param {State} state
 * @param {Object} operation
 * @return {State}
 */

function setNode(state, operation) {
  var path = operation.path;
  var properties = operation.properties;
  var _state9 = state;
  var document = _state9.document;

  var node = document.assertPath(path);
  node = node.merge(properties);
  document = document.updateDescendant(node);
  document = document.normalize();
  state = state.merge({ document: document });
  return state;
}

/**
 * Set `properties` on the selection.
 *
 * @param {State} state
 * @param {Object} operation
 * @return {State}
 */

function setSelection(state, operation) {
  var properties = operation.properties;
  var _state10 = state;
  var selection = _state10.selection;

  selection = selection.merge(properties);
  state = state.merge({ selection: selection });
  return state;
}

/**
 * Split a node by `path` at `offset`.
 *
 * @param {State} state
 * @param {Object} operation
 * @return {State}
 */

function splitNode(state, operation) {
  var path = operation.path;
  var offset = operation.offset;
  var _state11 = state;
  var document = _state11.document;

  var node = document.assertPath(path);
  var parent = document.getParent(node);
  var isParent = document == parent;
  var index = parent.nodes.indexOf(node);

  var child = node;
  var one = void 0;
  var two = void 0;

  if (node.kind != 'text') {
    child = node.getTextAtOffset(offset);
  }

  while (child && child != parent) {
    if (child.kind == 'text') {
      var i = node.kind == 'text' ? offset : offset - node.getOffset(child);
      var _child = child;
      var characters = _child.characters;

      var oneChars = characters.take(i);
      var twoChars = characters.skip(i);
      one = child.merge({ characters: oneChars });
      two = child.merge({ characters: twoChars, key: (0, _uid2.default)() });
    } else {
      var _child2 = child;
      var nodes = _child2.nodes;

      var oneNodes = nodes.takeUntil(function (n) {
        return n.key == one.key;
      }).push(one);
      var twoNodes = nodes.skipUntil(function (n) {
        return n.key == one.key;
      }).rest().unshift(two);
      one = child.merge({ nodes: oneNodes });
      two = child.merge({ nodes: twoNodes, key: (0, _uid2.default)() });
    }

    child = document.getParent(child);
  }

  parent = parent.removeNode(index);
  parent = parent.insertNode(index, two);
  parent = parent.insertNode(index, one);
  document = isParent ? parent : document.updateDescendant(parent);
  state = state.merge({ document: document });
  return state;
}